/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class CommandLine ---

CommandLine::CommandLine()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

CommandLine::~CommandLine()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

void CommandLine::declareOption(const String & name, int expectedParameters)
{
	ASSERTION_THIS;

	options.Delete(name);

	Option * option;
	if ((option = new Option(name)) == NULL)
		throw EOutOfMemory::Get();
	option->expectedParameters = expectedParameters;

	options.Set(option->name, option);
}

const ArrayList<String> * CommandLine::getOptionParameters(const String & optionName) const
{
	ASSERTION_THIS;

	Option * option = options.Get(optionName);
	return option != NULL ? &option->parameters : NULL;
}

const ArrayList<String> & CommandLine::getParameters() const
{
	ASSERTION_THIS;
	return parameters;
}

const String & CommandLine::getProgramName() const
{
	ASSERTION_THIS;
	return programName;
}

bool CommandLine::hasOption(const String & optionName) const
{
	ASSERTION_THIS;

	Option * option = options.Get(optionName);
	return option != NULL;
}

#ifdef _TOOLBOX_TEST
	int CommandLine::instanceCount = 0;
#endif

void CommandLine::readFrom(int argc, const char ** argv)
{
	ASSERTION_THIS;

	if (argc > 0)
		programName = argv[0];

	for (int argNo = 1; argNo < argc; argNo++)
		if (argv[argNo][0] == '/' || argv[argNo][0] == '-')
		{
			Option * option = options.Get(argv[argNo]);
			if (option != NULL)
			{
				option->parameters.DeleteAll();
				option->found = true;
				if (argNo + option->expectedParameters < argc)
				{
					for (int i=0; i<option->expectedParameters; i++)
					{
						argNo++;

						String * parameter;
						if ((parameter = new String(argv[argNo])) == NULL)
							throw EOutOfMemory::Get();
						option->parameters.Append(parameter);
					}
				}
				else
					argNo = argc;
			}
			else
			{
				if ((option = new Option(argv[argNo])) == NULL)
					throw EOutOfMemory::Get();
				option->found = true;
				options.Set(option->name, option);
			}
		}
		else
		{
			String * parameter;
			if ((parameter = new String(argv[argNo])) == NULL)
				throw EOutOfMemory::Get();
			parameters.Append(parameter);
		}
}

void CommandLine::readFromDefault()
{
	ASSERTION_THIS;
	readFrom(__argc, (const char **) __argv);
}

#ifdef _TOOLBOX_TEST
	void CommandLine::runTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class CommandLine::Option ---

CommandLine::Option::Option(const String & name)
{
	expectedParameters = 0;
	found = false;
	this->name = name;
}
