/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_HASHMAP_INL
#define __TOOLBOX_HASHMAP_INL


// --- Class HashMap ---

template <class VALUETYPE> HashMap<VALUETYPE>::HashMap()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(_TB_HM_DEFAULTINITSIZE, _TB_HM_DEFAULTDESIREDFILLRATIO);
}

template <class VALUETYPE> HashMap<VALUETYPE>::HashMap(long initSize)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(initSize, _TB_HM_DEFAULTDESIREDFILLRATIO);
}

template <class VALUETYPE> HashMap<VALUETYPE>::HashMap(long initSize, int desiredFillRatio)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(initSize, desiredFillRatio);
}

template <class VALUETYPE> HashMap<VALUETYPE>::~HashMap()
{
	ASSERTION_THIS;

	if (KeyBuckets)
	{
		DeleteAll();
		delete KeyBuckets;
		KeyBuckets = NULL;
	}

	if (ValueBuckets)
	{
		delete ValueBuckets;
		ValueBuckets = NULL;
	}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class VALUETYPE> void HashMap<VALUETYPE>::CheckSize()
{
	ASSERTION_THIS;

	int fillRatio = GetFillRatio();

	if (fillRatio < DesiredFillRatio / 2)
	{
		int newSize = Count * 100 / DesiredFillRatio;
		if (newSize < InitSize)
			newSize = InitSize;

		if (newSize < Size)
			Rehash(newSize);
	}
	else if (fillRatio > DesiredFillRatio * 2)
	{
		int newSize = Count * 100 / DesiredFillRatio;
		Rehash(newSize);
	}
}

template <class VALUETYPE> bool HashMap<VALUETYPE>::ContainsKey(void * key) const
{
	ASSERTION_THIS;

	int bucketNumber = HashCode(key) % Size;
	if (KeyBuckets[bucketNumber] == NULL)
		return false;

	PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
	while (KeyBucketItem != NULL && !Equals(KeyBucketItem->GetData(), key))
		KeyBucketItem = KeyBucketItem->GetNext();

	return KeyBucketItem != NULL;
}

template <class VALUETYPE> void * HashMap<VALUETYPE>::CreateKey(void * key) const
{
	ASSERTION_THIS;
	return key;
}

template <class VALUETYPE> void HashMap<VALUETYPE>::Delete(void * key)
{
	ASSERTION_THIS;

	VALUETYPE * Value = Unset(key);
	if (Value != NULL)
		delete Value;
}

template <class VALUETYPE> void HashMap<VALUETYPE>::DeleteAll()
{
	ASSERTION_THIS;

	for (int bucketNumber = 0; bucketNumber < Size; bucketNumber++)
	{
		if (KeyBuckets[bucketNumber] != NULL)
		{
			ASSERTION(ValueBuckets[bucketNumber] != NULL);

			PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
			PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
			while (KeyBucketItem != NULL)
			{
				ASSERTION(ValueBucketItem != NULL);

				DestroyKey(KeyBucketItem->GetData());
				delete (VALUETYPE *) ValueBucketItem->GetData();

				KeyBucketItem = KeyBucketItem->GetNext();
				ValueBucketItem = ValueBucketItem->GetNext();
			}
			ASSERTION(ValueBucketItem == NULL);

			delete KeyBuckets[bucketNumber]; KeyBuckets[bucketNumber] = NULL;
			delete ValueBuckets[bucketNumber]; ValueBuckets[bucketNumber] = NULL;
		}

		ASSERTION(KeyBuckets[bucketNumber] == NULL);
		ASSERTION(ValueBuckets[bucketNumber] == NULL);
	}

	Count = 0;
	CheckSize();
}

template <class VALUETYPE> void HashMap<VALUETYPE>::DestroyKey(void * key) const
{
	ASSERTION_THIS;
}

template <class VALUETYPE> bool HashMap<VALUETYPE>::Equals(void * key1, void * key2) const
{
	ASSERTION_THIS;
	return key1 == key2;
}

template <class VALUETYPE> VALUETYPE * HashMap<VALUETYPE>::Get(void * key) const
{
	ASSERTION_THIS;

	int bucketNumber = HashCode(key) % Size;
	if (KeyBuckets[bucketNumber] == NULL)
		return NULL;
	ASSERTION(ValueBuckets[bucketNumber] != NULL);

	PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
	PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
	while (KeyBucketItem != NULL && !Equals(KeyBucketItem->GetData(), key))
	{
		ASSERTION(ValueBucketItem != NULL);
		KeyBucketItem = KeyBucketItem->GetNext();
		ValueBucketItem = ValueBucketItem->GetNext();
	}

	return ValueBucketItem != NULL ? (VALUETYPE *) ValueBucketItem->GetData() : NULL;
}

template <class VALUETYPE> inline long HashMap<VALUETYPE>::GetCount() const
{
	ASSERTION_THIS;
	return Count;
}

template <class VALUETYPE> inline int HashMap<VALUETYPE>::GetDesiredFillRatio() const
{
	ASSERTION_THIS;
	return DesiredFillRatio;
}

template <class VALUETYPE> inline int HashMap<VALUETYPE>::GetFillRatio() const
{
	ASSERTION_THIS;
	return Count * 100 / Size;
}

template <class VALUETYPE> PointeredList * HashMap<VALUETYPE>::GetKeys() const
{
	ASSERTION_THIS;

	PointeredList * KeysList;
	if ((KeysList = new PointeredList) == NULL)
		throw EOutOfMemory::Get();

	for (int bucketNumber = 0; bucketNumber < Size; bucketNumber++)
		if (KeyBuckets[bucketNumber] != NULL)
		{
			PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
			while (KeyBucketItem != NULL)
			{
				PointeredListItem * KeysListItem;
				if ((KeysListItem = new PointeredListItem) == NULL)
					throw EOutOfMemory::Get();

				KeysListItem->SetData(KeyBucketItem->GetData());
				KeysList->Append(KeysListItem);

				KeyBucketItem = KeyBucketItem->GetNext();
			}
		}

	return KeysList;
}

template <class VALUETYPE> inline long HashMap<VALUETYPE>::GetSize() const
{
	ASSERTION_THIS;
	return Size;
}

template <class VALUETYPE> PointeredList * HashMap<VALUETYPE>::GetValues() const
{
	ASSERTION_THIS;

	PointeredList * ValuesList;
	if ((ValuesList = new PointeredList) == NULL)
		throw EOutOfMemory::Get();

	for (int bucketNumber = 0; bucketNumber < Size; bucketNumber++)
		if (ValueBuckets[bucketNumber] != NULL)
		{
			PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
			while (ValueBucketItem != NULL)
			{
				PointeredListItem * ValuesListItem;
				if ((ValuesListItem = new PointeredListItem) == NULL)
					throw EOutOfMemory::Get();

				ValuesListItem->SetData(ValueBucketItem->GetData());
				ValuesList->Append(ValuesListItem);

				ValueBucketItem = ValueBucketItem->GetNext();
			}
		}

	return ValuesList;
}

template <class VALUETYPE> unsigned long HashMap<VALUETYPE>::HashCode(void * key) const
{
	ASSERTION_THIS;

	return ((unsigned long) key) / 8UL * 1009UL;
}

template <class VALUETYPE> void HashMap<VALUETYPE>::Init(long initSize, int desiredFillRatio)
{
	ASSERTION_THIS;

	if (initSize < _TB_HM_MINIMUMSIZE)
		initSize = _TB_HM_DEFAULTINITSIZE;

	if (desiredFillRatio < _TB_HM_MINIMUMDESIREDFILLRATIO)
		desiredFillRatio = _TB_HM_DEFAULTDESIREDFILLRATIO;

	Count = 0;
	DesiredFillRatio = desiredFillRatio;
	InitSize = initSize;
	KeyBuckets = NULL;
	Size = 0;
	ValueBuckets = NULL;

	Rehash(initSize);
}

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> int HashMap<VALUETYPE>::InstanceCount = 0;
#endif

template <class VALUETYPE> inline bool HashMap<VALUETYPE>::IsEmpty() const
{
	return Count <= 0;
}

template <class VALUETYPE> void HashMap<VALUETYPE>::Rehash(long newSize)
{
	ASSERTION_THIS;

	PointeredList ** NewKeyBuckets;
	if ((NewKeyBuckets = new PointeredList * [newSize]) == NULL)
		throw EOutOfMemory::Get();

	PointeredList ** NewValueBuckets;
	if ((NewValueBuckets = new PointeredList * [newSize]) == NULL)
		throw EOutOfMemory::Get();

	for (long i=0; i<newSize; i++)
	{
		NewKeyBuckets[i] = NULL;
		NewValueBuckets[i] = NULL;
	}

	PointeredList ** OldKeyBuckets = KeyBuckets;
	PointeredList ** OldValueBuckets = ValueBuckets;
	long oldSize = Size;
	KeyBuckets = NewKeyBuckets;
	ValueBuckets = NewValueBuckets;
	Size = newSize;

	// Since we build up the buckets again, the Count must be reset.
	Count = 0;

	if (OldKeyBuckets != NULL && OldValueBuckets != NULL)
		for (int bucketNumber = 0; bucketNumber < oldSize; bucketNumber++)
			if (OldKeyBuckets[bucketNumber] != NULL)
			{
				ASSERTION(OldValueBuckets[bucketNumber] != NULL);

				PointeredListItem * KeyBucketItem = OldKeyBuckets[bucketNumber]->GetFirst();
				PointeredListItem * ValueBucketItem = OldValueBuckets[bucketNumber]->GetFirst();
				while (KeyBucketItem != NULL)
				{
					ASSERTION(ValueBucketItem != NULL);
					SetWithoutSizeChecking(KeyBucketItem->GetData(), (VALUETYPE *) ValueBucketItem->GetData());
					KeyBucketItem = KeyBucketItem->GetNext();
					ValueBucketItem = ValueBucketItem->GetNext();
				}

				delete OldKeyBuckets[bucketNumber];
				delete OldValueBuckets[bucketNumber];
			}

	if (OldKeyBuckets != NULL)
		delete OldKeyBuckets;
	if (OldValueBuckets != NULL)
		delete OldValueBuckets;
}

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> void HashMap<VALUETYPE>::RunTestSuite(int * performedTests, int * failedTests)
	{
		HashMap<TestObject> * TestHashMap = new HashMap<TestObject>;
		TestObject * Keys[400];
		TestObject * Values[400];

		if (TestObject::GetInstanceCount() != 0) {printf("Test HashMap[2] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestHashMap->GetCount() != 0) {printf("Test HashMap[3] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (int i=0; i<400; i++)
		{
			Keys[i] = new TestObject;
			Keys[i]->Number = 1000+i;
			Values[i] = new TestObject;
			Values[i]->Number = 2000+i;
		}

		if (TestObject::GetInstanceCount() != 800) {printf("Test HashMap[4] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=0; i<200; i++)
		{
			TestHashMap->Set(Keys[i], Values[i]);
			RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, 0, i+1);
		}

		for (i=199; i>=100; i--)
		{
			TestHashMap->Unset(Keys[i]);
			RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, 0, i);
		}

		if (TestObject::GetInstanceCount() != 800) {printf("Test HashMap[5] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=100; i<400; i++)
		{
			TestHashMap->Set(Keys[i], Values[i]);
			RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, 0, i+1);
		}

		for (i=399; i>=300; i--)
		{
			TestHashMap->Unset(Keys[i]);
			RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, 0, i);
		}

		if (TestObject::GetInstanceCount() != 800) {printf("Test HashMap[6] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=0; i<100; i++)
		{
			TestHashMap->Delete(Keys[i]);
			RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, i+1, 300);
		}

		if (TestObject::GetInstanceCount() != 700) {printf("Test HashMap[7] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestHashMap->DeleteAll();
		RunTestSuiteCheck(performedTests, failedTests, TestHashMap, Keys, Values, 0, 0);

		if (TestHashMap->GetCount() != 0) {printf("Test HashMap[8] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		delete TestHashMap;
		if (TestObject::GetInstanceCount() != 500) {printf("Test HashMap[9] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=0; i<400; i++)
			delete Keys[i];
		for (i=300; i<400; i++)
			delete Values[i];

		if (TestObject::GetInstanceCount() != 0) {printf("Test HashMap[10] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
	}
#endif

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> void HashMap<VALUETYPE>::RunTestSuiteCheck(int * performedTests,
		int * failedTests, HashMap<TestObject> * TestHashMap, TestObject ** Keys, TestObject ** Values,
		int firstContained, int firstNotContained)
	{
		if (TestHashMap->GetCount() != firstNotContained-firstContained)
			{printf("Test HashMap[11] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (int i=0; i<firstContained; i++)
		{
			if (TestHashMap->Get(Keys[i]) != NULL || TestHashMap->ContainsKey(Keys[i]))
				break;
		}
		if (i<firstContained) {printf("Test HashMap[12] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=firstContained; i<firstNotContained; i++)
		{
			if (TestHashMap->Get(Keys[i]) == NULL || !TestHashMap->ContainsKey(Keys[i]))
				break;
		}
		if (i<firstContained) {printf("Test HashMap[13] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		for (i=firstNotContained; i<400; i++)
		{
			if (TestHashMap->Get(Keys[i]) != NULL || TestHashMap->ContainsKey(Keys[i]))
				break;
		}
		if (i<firstContained) {printf("Test HashMap[14] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		PointeredList * TestKeys = TestHashMap->GetKeys();
		if (TestKeys->GetCount() != firstNotContained-firstContained)
			{printf("Test HashMap[15] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		PointeredListItem * CheckItem = TestKeys->GetFirst();
		while (CheckItem != NULL)
		{
			PointeredListItem * CheckItem2 = CheckItem->GetNext();
			while (CheckItem2 != NULL)
			{
				if (CheckItem->GetData() == NULL || CheckItem2->GetData() == NULL
					|| CheckItem->GetData() == CheckItem2->GetData())
					break;

				CheckItem2 = CheckItem2->GetNext();
			}
			if (CheckItem2 != NULL)
				break;

			CheckItem = CheckItem->GetNext();
		}
		if (CheckItem != NULL) {printf("Test HashMap[16] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		CheckItem = TestKeys->GetFirst();
		while (CheckItem != NULL)
		{
			int number = ((TestObject *) CheckItem->GetData())->Number;
			if (number < 1000+firstContained || number >= 1000+firstNotContained)
				break;

			CheckItem = CheckItem->GetNext();
		}
		if (CheckItem != NULL) {printf("Test HashMap[17] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete TestKeys;

		PointeredList * TestValues = TestHashMap->GetValues();
		if (TestValues->GetCount() != firstNotContained-firstContained)
			{printf("Test HashMap[16] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		CheckItem = TestValues->GetFirst();
		while (CheckItem != NULL)
		{
			PointeredListItem * CheckItem2 = CheckItem->GetNext();
			while (CheckItem2 != NULL)
			{
				if (CheckItem->GetData() == NULL || CheckItem2->GetData() == NULL
					|| CheckItem->GetData() == CheckItem2->GetData())
					break;

				CheckItem2 = CheckItem2->GetNext();
			}
			if (CheckItem2 != NULL)
				break;

			CheckItem = CheckItem->GetNext();
		}
		if (CheckItem != NULL) {printf("Test HashMap[18] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		CheckItem = TestValues->GetFirst();
		while (CheckItem != NULL)
		{
			int number = ((TestObject *) CheckItem->GetData())->Number;
			if (number < 2000+firstContained || number >= 2000+firstNotContained)
				break;

			CheckItem = CheckItem->GetNext();
		}
		if (CheckItem != NULL) {printf("Test HashMap[19] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete TestValues;
	}
#endif

template <class VALUETYPE> void HashMap<VALUETYPE>::Set(void * key, VALUETYPE * Value)
{
	ASSERTION_THIS;

	SetWithoutSizeChecking(CreateKey(key), Value);
	CheckSize();
}

template <class VALUETYPE> void HashMap<VALUETYPE>::SetWithoutSizeChecking(void * key, VALUETYPE * Value)
{
	ASSERTION_THIS;

	// Find bucket number for key
	int bucketNumber = HashCode(key) % Size;

	// Ensure bucket exists
	if (KeyBuckets[bucketNumber] == NULL)
	{
		ASSERTION(ValueBuckets[bucketNumber] == NULL);
		if ((KeyBuckets[bucketNumber] = new PointeredList) == NULL)
			throw EOutOfMemory::Get();
		if ((ValueBuckets[bucketNumber] = new PointeredList) == NULL)
			throw EOutOfMemory::Get();
	}

	// Search existing key
	PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
	PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
	while (KeyBucketItem != NULL && !Equals(KeyBucketItem->GetData(), key))
	{
		ASSERTION(ValueBucketItem != NULL);
		KeyBucketItem = KeyBucketItem->GetNext();
		ValueBucketItem = ValueBucketItem->GetNext();
	}

	if (KeyBucketItem != NULL)
	{
		// Key exists, replace value and destroy key
		DestroyKey(key);
		delete (VALUETYPE *) ValueBucketItem->GetData();
		ValueBucketItem->SetData(Value);
	}
	else
	{
		// Key does not exist, create key and extend bucket
		if ((KeyBucketItem = new PointeredListItem) == NULL)
			throw EOutOfMemory::Get();
		KeyBucketItem->SetData(key);
		KeyBuckets[bucketNumber]->Append(KeyBucketItem);

		// Now add value
		if ((ValueBucketItem = new PointeredListItem) == NULL)
			throw EOutOfMemory::Get();
		ValueBucketItem->SetData(Value);
		ValueBuckets[bucketNumber]->Append(ValueBucketItem);

		Count++;
	}
}

template <class VALUETYPE> VALUETYPE * HashMap<VALUETYPE>::Unset(void * key)
{
	ASSERTION_THIS;

	int bucketNumber = HashCode(key) % Size;
	if (KeyBuckets[bucketNumber] == NULL)
		return NULL;

	PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
	PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
	while (KeyBucketItem != NULL && !Equals(KeyBucketItem->GetData(), key))
	{
		ASSERTION(ValueBucketItem != NULL);
		KeyBucketItem = KeyBucketItem->GetNext();
		ValueBucketItem = ValueBucketItem->GetNext();
	}

	if (KeyBucketItem == NULL)
		return NULL;

	DestroyKey(KeyBucketItem->GetData());
	KeyBuckets[bucketNumber]->Delete(KeyBucketItem);

	VALUETYPE * OldValue = (VALUETYPE *) ValueBucketItem->GetData();
	ValueBuckets[bucketNumber]->Delete(ValueBucketItem);

	if (KeyBuckets[bucketNumber]->IsEmpty())
	{
		ASSERTION(ValueBuckets[bucketNumber]->IsEmpty());

		delete KeyBuckets[bucketNumber]; KeyBuckets[bucketNumber] = NULL;
		delete ValueBuckets[bucketNumber]; ValueBuckets[bucketNumber] = NULL;
	}

	Count--;
	return OldValue;
}

template <class VALUETYPE> void HashMap<VALUETYPE>::UnsetAll()
{
	for (int bucketNumber = 0; bucketNumber < Size; bucketNumber++)
	{
		if (KeyBuckets[bucketNumber] != NULL)
		{
			ASSERTION(ValueBuckets[bucketNumber] != NULL);

			PointeredListItem * KeyBucketItem = KeyBuckets[bucketNumber]->GetFirst();
			PointeredListItem * ValueBucketItem = ValueBuckets[bucketNumber]->GetFirst();
			while (KeyBucketItem != NULL)
			{
				ASSERTION(ValueBucketItem != NULL);

				DestroyKey(KeyBucketItem->GetData());

				KeyBucketItem = KeyBucketItem->GetNext();
				ValueBucketItem = ValueBucketItem->GetNext();
			}
			ASSERTION(ValueBucketItem == NULL);

			delete KeyBuckets[bucketNumber]; KeyBuckets[bucketNumber] = NULL;
			delete ValueBuckets[bucketNumber]; ValueBuckets[bucketNumber] = NULL;
		}

		ASSERTION(KeyBuckets[bucketNumber] == NULL);
		ASSERTION(ValueBuckets[bucketNumber] == NULL);
	}

	Count = 0;
	CheckSize();
}


// --- Class IntKeyHashMap ---

template <class VALUETYPE> IntKeyHashMap<VALUETYPE>::IntKeyHashMap()
	: HashMap<VALUETYPE>()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> IntKeyHashMap<VALUETYPE>::IntKeyHashMap(long initSize)
	: HashMap<VALUETYPE>(initSize)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> IntKeyHashMap<VALUETYPE>::IntKeyHashMap
	(long initSize, int desiredFillRatio)
	: HashMap<VALUETYPE>(initSize, desiredFillRatio)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> IntKeyHashMap<VALUETYPE>::~IntKeyHashMap()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class VALUETYPE> inline bool IntKeyHashMap<VALUETYPE>::ContainsKey(long key) const
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::ContainsKey((void *) key);
}

template <class VALUETYPE> inline void IntKeyHashMap<VALUETYPE>::Delete(long key)
{
	ASSERTION_THIS;
	HashMap<VALUETYPE>::Delete((void *) key);
}

template <class VALUETYPE> inline VALUETYPE * IntKeyHashMap<VALUETYPE>::Get(long key) const
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::Get((void *) key);
}

template <class VALUETYPE> unsigned long IntKeyHashMap<VALUETYPE>::HashCode(void * key) const
{
	ASSERTION_THIS;

	unsigned long value = (long) key;
	return value * 1009UL;
}

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> int IntKeyHashMap<VALUETYPE>::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
template <class VALUETYPE> void IntKeyHashMap<VALUETYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif

template <class VALUETYPE> inline void IntKeyHashMap<VALUETYPE>::Set(long key, VALUETYPE * Value)
{
	ASSERTION_THIS;
	HashMap<VALUETYPE>::Set((void *) key, Value);
}

template <class VALUETYPE> inline VALUETYPE * IntKeyHashMap<VALUETYPE>::Unset(long key)
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::Unset((void *) key);
}


// --- Class StringKeyHashMap ---

template <class VALUETYPE> StringKeyHashMap<VALUETYPE>::StringKeyHashMap()
	: HashMap<VALUETYPE>()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> StringKeyHashMap<VALUETYPE>::StringKeyHashMap(long initSize)
	: HashMap<VALUETYPE>(initSize)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> StringKeyHashMap<VALUETYPE>::StringKeyHashMap
	(long initSize, int desiredFillRatio)
	: HashMap<VALUETYPE>(initSize, desiredFillRatio)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> StringKeyHashMap<VALUETYPE>::~StringKeyHashMap()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif

	// The keys have to be deleted already here, because the overridden DestroyKey method
	// is not used when the super class destructor deletes the keys.
	if (KeyBuckets != NULL)
		DeleteAll();
}

template <class VALUETYPE> inline bool StringKeyHashMap<VALUETYPE>::ContainsKey(const char * key) const
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::ContainsKey((void *) key);
}

template <class VALUETYPE> void * StringKeyHashMap<VALUETYPE>::CreateKey(void * key) const
{
	ASSERTION_THIS;

	char * bucketKey;
	if ((bucketKey = new char[strlen((char *) key) + 1]) == NULL)
		throw EOutOfMemory::Get();

	strcpy(bucketKey, (char *) key);
	return bucketKey;
}

template <class VALUETYPE> inline void StringKeyHashMap<VALUETYPE>::Delete(const char * key)
{
	ASSERTION_THIS;
	HashMap<VALUETYPE>::Delete((void *) key);
}

template <class VALUETYPE> void StringKeyHashMap<VALUETYPE>::DestroyKey(void * key) const
{
	ASSERTION_THIS;
	delete (char *) key;
}

template <class VALUETYPE> bool StringKeyHashMap<VALUETYPE>::Equals(void * key1, void * key2) const
{
	ASSERTION_THIS;
	return key1 != NULL && key2 != NULL && strcmp((const char *) key1, (const char *) key2) == 0;
}

template <class VALUETYPE> inline VALUETYPE * StringKeyHashMap<VALUETYPE>::Get(const char * key) const
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::Get((void *) key);
}

template <class VALUETYPE> unsigned long StringKeyHashMap<VALUETYPE>::HashCode(void * key) const
{
	ASSERTION_THIS;

	char * string = (char *) key;
	int len = strlen(string);
	int step = len/16 >= 1 ? len/16 : 1;
	unsigned long hashCode = 0;

	for (int i=0; i<len; i+=step)
		hashCode += i%2 == 0 ? string[i]*11 : string[i]*7;

	return hashCode;
}

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> int StringKeyHashMap<VALUETYPE>::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
template <class VALUETYPE> void StringKeyHashMap<VALUETYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif

template <class VALUETYPE> inline void StringKeyHashMap<VALUETYPE>::Set(const char * key, VALUETYPE * Value)
{
	ASSERTION_THIS;
	HashMap<VALUETYPE>::Set((void *) key, Value);
}

template <class VALUETYPE> inline VALUETYPE * StringKeyHashMap<VALUETYPE>::Unset(const char * key)
{
	ASSERTION_THIS;
	return HashMap<VALUETYPE>::Unset((void *) key);
}


// --- Class ICStringKeyHashMap ---

template <class VALUETYPE> ICStringKeyHashMap<VALUETYPE>::ICStringKeyHashMap()
	: StringKeyHashMap<VALUETYPE>()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> ICStringKeyHashMap<VALUETYPE>::ICStringKeyHashMap(long initSize)
	: StringKeyHashMap<VALUETYPE>(initSize)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> ICStringKeyHashMap<VALUETYPE>::ICStringKeyHashMap
	(long initSize, int desiredFillRatio)
	: StringKeyHashMap<VALUETYPE>(initSize, desiredFillRatio)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

template <class VALUETYPE> ICStringKeyHashMap<VALUETYPE>::~ICStringKeyHashMap()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class VALUETYPE> bool ICStringKeyHashMap<VALUETYPE>::Equals(void * key1, void * key2) const
{
	ASSERTION_THIS;
	return key1 != NULL && key2 != NULL && stricmp((const char *) key1, (const char *) key2) == 0;
}

template <class VALUETYPE> unsigned long ICStringKeyHashMap<VALUETYPE>::HashCode(void * key) const
{
	ASSERTION_THIS;

	char * string = (char *) key;
	int len = strlen(string);
	int step = len/16 >= 1 ? len/16 : 1;
	unsigned long hashCode = 0;

	for (int i=0; i<len; i+=step)
		hashCode += i%2 == 0 ? String::ToLowerCase(string[i])*11 : String::ToLowerCase(string[i])*7;

	return hashCode;
}

#ifdef _TOOLBOX_TEST
	template <class VALUETYPE> int ICStringKeyHashMap<VALUETYPE>::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
template <class VALUETYPE> void ICStringKeyHashMap<VALUETYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif


#endif
