/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_MD5HASHER_INL
#define __TOOLBOX_MD5HASHER_INL


// --- Class MD5Hasher ---

inline void MD5Hasher::hash(unsigned char number)
{
	hash(&number, 1);
}

inline void MD5Hasher::hash(unsigned int number)
{
	unsigned char data[2];

	for (int i=0; i<2; i++)
	{
		data[i] = number & 0xFF;
		number >>= 8;
	}

	hash(data, 2);
}

inline void MD5Hasher::hash(unsigned long number)
{
	unsigned char data[4];

	for (int i=0; i<4; i++)
	{
		data[i] = (unsigned char) (number & 0xFF);
		number >>= 8;
	}

	hash(data, 4);
}

void MD5Hasher::transform(unsigned long * newState)
{
	unsigned long a = state[0];
	unsigned long b = state[1];
	unsigned long c = state[2];
	unsigned long d = state[3];

	_TB_MD5HASHER_FF(a, b, c, d, newState[0], 7, 3614090360);
	_TB_MD5HASHER_FF(d, a, b, c, newState[1], 12, 3905402710);
	_TB_MD5HASHER_FF(c, d, a, b, newState[2], 17,  606105819);
	_TB_MD5HASHER_FF(b, c, d, a, newState[3], 22, 3250441966);
	_TB_MD5HASHER_FF(a, b, c, d, newState[4], 7, 4118548399);
	_TB_MD5HASHER_FF(d, a, b, c, newState[5], 12, 1200080426);
	_TB_MD5HASHER_FF(c, d, a, b, newState[6], 17, 2821735955);
	_TB_MD5HASHER_FF(b, c, d, a, newState[7], 22, 4249261313);
	_TB_MD5HASHER_FF(a, b, c, d, newState[8], 7, 1770035416);
	_TB_MD5HASHER_FF(d, a, b, c, newState[9], 12, 2336552879);
	_TB_MD5HASHER_FF(c, d, a, b, newState[10], 17, 4294925233);
	_TB_MD5HASHER_FF(b, c, d, a, newState[11], 22, 2304563134);
	_TB_MD5HASHER_FF(a, b, c, d, newState[12], 7, 1804603682);
	_TB_MD5HASHER_FF(d, a, b, c, newState[13], 12, 4254626195);
	_TB_MD5HASHER_FF(c, d, a, b, newState[14], 17, 2792965006);
	_TB_MD5HASHER_FF(b, c, d, a, newState[15], 22, 1236535329);

	_TB_MD5HASHER_GG(a, b, c, d, newState[1], 5, 4129170786);
	_TB_MD5HASHER_GG(d, a, b, c, newState[6], 9, 3225465664);
	_TB_MD5HASHER_GG(c, d, a, b, newState[11], 14,  643717713);
	_TB_MD5HASHER_GG(b, c, d, a, newState[0], 20, 3921069994);
	_TB_MD5HASHER_GG(a, b, c, d, newState[5], 5, 3593408605);
	_TB_MD5HASHER_GG(d, a, b, c, newState[10], 9,   38016083);
	_TB_MD5HASHER_GG(c, d, a, b, newState[15], 14, 3634488961);
	_TB_MD5HASHER_GG(b, c, d, a, newState[4], 20, 3889429448);
	_TB_MD5HASHER_GG(a, b, c, d, newState[9], 5,  568446438);
	_TB_MD5HASHER_GG(d, a, b, c, newState[14], 9, 3275163606);
	_TB_MD5HASHER_GG(c, d, a, b, newState[3], 14, 4107603335);
	_TB_MD5HASHER_GG(b, c, d, a, newState[8], 20, 1163531501);
	_TB_MD5HASHER_GG(a, b, c, d, newState[13], 5, 2850285829);
	_TB_MD5HASHER_GG(d, a, b, c, newState[2], 9, 4243563512);
	_TB_MD5HASHER_GG(c, d, a, b, newState[7], 14, 1735328473);
	_TB_MD5HASHER_GG(b, c, d, a, newState[12], 20, 2368359562);

	_TB_MD5HASHER_HH(a, b, c, d, newState[5], 4, 4294588738);
	_TB_MD5HASHER_HH(d, a, b, c, newState[8], 11, 2272392833);
	_TB_MD5HASHER_HH(c, d, a, b, newState[11], 16, 1839030562);
	_TB_MD5HASHER_HH(b, c, d, a, newState[14], 23, 4259657740);
	_TB_MD5HASHER_HH(a, b, c, d, newState[1], 4, 2763975236);
	_TB_MD5HASHER_HH(d, a, b, c, newState[4], 11, 1272893353);
	_TB_MD5HASHER_HH(c, d, a, b, newState[7], 16, 4139469664);
	_TB_MD5HASHER_HH(b, c, d, a, newState[10], 23, 3200236656);
	_TB_MD5HASHER_HH(a, b, c, d, newState[13], 4,  681279174);
	_TB_MD5HASHER_HH(d, a, b, c, newState[0], 11, 3936430074);
	_TB_MD5HASHER_HH(c, d, a, b, newState[3], 16, 3572445317);
	_TB_MD5HASHER_HH(b, c, d, a, newState[6], 23,   76029189);
	_TB_MD5HASHER_HH(a, b, c, d, newState[9], 4, 3654602809);
	_TB_MD5HASHER_HH(d, a, b, c, newState[12], 11, 3873151461);
	_TB_MD5HASHER_HH(c, d, a, b, newState[15], 16,  530742520);
	_TB_MD5HASHER_HH(b, c, d, a, newState[2], 23, 3299628645);

	_TB_MD5HASHER_II(a, b, c, d, newState[0], 6, 4096336452);
	_TB_MD5HASHER_II(d, a, b, c, newState[7], 10, 1126891415);
	_TB_MD5HASHER_II(c, d, a, b, newState[14], 15, 2878612391);
	_TB_MD5HASHER_II(b, c, d, a, newState[5], 21, 4237533241);
	_TB_MD5HASHER_II(a, b, c, d, newState[12], 6, 1700485571);
	_TB_MD5HASHER_II(d, a, b, c, newState[3], 10, 2399980690);
	_TB_MD5HASHER_II(c, d, a, b, newState[10], 15, 4293915773);
	_TB_MD5HASHER_II(b, c, d, a, newState[1], 21, 2240044497);
	_TB_MD5HASHER_II(a, b, c, d, newState[8], 6, 1873313359);
	_TB_MD5HASHER_II(d, a, b, c, newState[15], 10, 4264355552);
	_TB_MD5HASHER_II(c, d, a, b, newState[6], 15, 2734768916);
	_TB_MD5HASHER_II(b, c, d, a, newState[13], 21, 1309151649);
	_TB_MD5HASHER_II(a, b, c, d, newState[4], 6, 4149444226);
	_TB_MD5HASHER_II(d, a, b, c, newState[11], 10, 3174756917);
	_TB_MD5HASHER_II(c, d, a, b, newState[2], 15,  718787259);
	_TB_MD5HASHER_II(b, c, d, a, newState[9], 21, 3951481745);

	state[0] += a;
	state[1] += b;
	state[2] += c;
	state[3] += d;
}


#endif
